/**********************************************************************
 * vanessa_mcast.h                                         October 2002
 * Horms                                             horms@verge.net.au
 *
 * Encapsulation of a Multicast Group
 *
 * libvaness_mcast
 * Multicast Helper Library
 * Copyright (C) 2000 Horms
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 *
 **********************************************************************/


#ifndef _VANESSA_MCAST_H
#define _VANESSA_MCAST_H


#include <netinet/in.h>

typedef struct {
	int sk;
	struct sockaddr_in sa_in;
} vanessa_mcast_t;

/**********************************************************************
 * Macros to access elements of a handle
 **********************************************************************/

#define VANESSA_MCAST_GET_SOCKET(_hdl) ((_hdl)->sk)
#define VANESSA_MCAST_GET_SA_IN(_hdl) ((_hdl)->sa_in)


/**********************************************************************
 * vanessa_mcast_create
 * Create a vanessa_mcast_t
 *
 * pre: none
 * post: new vanessa_mcast_t is allocated and initialised
 * return: pointer to new vanessa_mcast_t
 **********************************************************************/

vanessa_mcast_t *vanessa_mcast_create(void);


/**********************************************************************
 * vanessa_mcast_destroy
 * Destroy a vanessa_mcast_t
 *
 * pre: hdl: vanessa_mcast_t to destroy
 * post: hdl is freed
 * return: none
 **********************************************************************/

void vanessa_mcast_destroy(vanessa_mcast_t *hdl);


/**********************************************************************
 * vanessa_mcast_open_server
 * vanessa_mcast_t for listening as a server
 *
 * pre: group: group to open
 *      port: port to use
 *      ifname: interface to listen on
 *      flags: flags passed to vanessa_mcast_open()
 * post: Handle is created
 *       Socket is opened
 *       Socket is set as reusable
 *       Socket is bound to group ad port
 *       Socket joins Group
 *          If ifname is set, then it joins the group on that interface
 *          Else on the default interface as per the kernel
 * return: new handle
 *         NULL on error
 **********************************************************************/

vanessa_mcast_t *vanessa_mcast_open_server(const char *group, const char *port, 
		const char *ifname, unsigned int flags);


/**********************************************************************
 * vanessa_mcast_open_client
 * vanessa_mcast_t for listening as a server
 *
 * pre: group: group to open
 *      port: port to use
 *      ifname: interface to listen on
 *      ttl: Time to live for outoging dagrams (in hops)
 *      loopback: Receive packets from sent by this host
 *                0 for no, 1 for yes
 *      flags: flags passed to vanessa_mcast_open()
 * post: Handle is created
 *       Socket is opened
 *       Set loopback option for socket
 *       Set ttl for socket
 * return: new handle
 *         NULL on error
 **********************************************************************/

vanessa_mcast_t *vanessa_mcast_open_client(const char *group, const char *port,
                const char *ifname, unsigned char ttl, unsigned int loopback,
                unsigned int flags);


/**********************************************************************
 * vanessa_mcast_close
 * Close a handle by closing its file descriptor and freeing its memory
 *
 * pre: hdl: handle to close
 * return: 0 on success
 *         < 0 on error, as per close(2)
 **********************************************************************/

int vanessa_mcast_close(vanessa_mcast_t *hdl);


/**********************************************************************
 * vanessa_mcast_sendto
 * Send data to a handle, analoglous to sendto(2)
 *
 * pre: hdl: handle to send data to
 *      msg: message to send
 *      len: length of message in bytes
 *      flags: passed to sendto(2)
 * post: sendto(2) will be called repeatedly until either
 *       len bytes of msg is sent, or an error occurs
 * return: len on success
 *         < 0, as per sendto(2)
 **********************************************************************/

int vanessa_mcast_sendto(vanessa_mcast_t *hdl, const void *msg, size_t len, 
		int flags);


/**********************************************************************
 * vanessa_mcast_send
 * Send data to a handle, analoglous to send(2)
 *
 * pre: hdl: handle to send data to
 *      msg: message to send
 *      len: length of message in bytes
 *      flags: passed to send(2)
 * post: send(2) will be called repeatedly until either
 *       len bytes of msg is sent, or an error occurs
 * return: len on success
 *         < 0, as per send(2)
 **********************************************************************/

int vanessa_mcast_send(vanessa_mcast_t *hdl, const void *msg, size_t len, 
		int flags);


/**********************************************************************
 * vanessa_mcast_recvfrom
 * Receive data from a handle, analoglous to recvfrom(2)
 *
 * pre: hdl: handle to receive data from
 *      msg: buffer to receive message into
 *      len: length of message in bytes
 *      flags: passed to recvfrom(2)
 *      from: handle to recieve from
 *            may be NULL, in which case behaviour is the
 *            same as vanessa_mcast_recvfrom()
 * post: recvfrom(2) is called once to read data
 * return: number of bytes read
 *         < 0, as per recvfrom(2)
 **********************************************************************/

int vanessa_mcast_recvfrom(vanessa_mcast_t *hdl, void *msg, size_t len, 
		int flags, vanessa_mcast_t *from);


/**********************************************************************
 * vanessa_mcast_recvfrom
 * Receive data from a handle, analoglous to recv(2)
 *
 * pre: hdl: handle to receive data from
 *      msg: buffer to receive message into
 *      len: length of message in bytes
 *      flags: passed to recvfrom(2)
 * post: recvfrom(2) is called once to read data
 * return: number of bytes read
 *         < 0, as per recvfrom(2)
 **********************************************************************/
int vanessa_mcast_recv(vanessa_mcast_t *hdl, void *msg, size_t len, int flags);


/**********************************************************************
 * vanessa_mcast_group_test
 * Test to see if an alleged group is in fact a multicast group
 *
 * pre: group: alleged group: may be an ip address or hostname
 *      flag: passed to vanessa_socket_host_in_addr
 * return: 1 if group is a multicast grou[
 *         0 if it is not
 *         -1 on error
 **********************************************************************/

int vanessa_mcast_group_test(const char *group, unsigned int flag);


/**********************************************************************
 * vanessa_mcast_set_iface
 * set sending interface of vanessa_mcast.
 *
 * pre: hdl: handle to set sending interface
 *      ifname: interface name to be set to the handle
 * return: 0 on success
 *         <0 on error
 **********************************************************************/

int vanessa_mcast_set_iface(vanessa_mcast_t *hdl, const char *ifname);

#endif /* _VANESSA_MCAST_H */
